@@ -1,31 +1,31 @@
 /*
- * Copyright (c) 2002-2007 JGoodies Karsten Lentzsch. All Rights Reserved.
+ * Copyright (c) 2002-2008 JGoodies Karsten Lentzsch. All Rights Reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
+ * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- * 
- *  o Redistributions of source code must retain the above copyright notice, 
- *    this list of conditions and the following disclaimer. 
- *     
- *  o Redistributions in binary form must reproduce the above copyright notice, 
- *    this list of conditions and the following disclaimer in the documentation 
- *    and/or other materials provided with the distribution. 
- *     
- *  o Neither the name of JGoodies Karsten Lentzsch nor the names of 
- *    its contributors may be used to endorse or promote products derived 
- *    from this software without specific prior written permission. 
- *     
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
- * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
- * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ *
+ *  o Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ *  o Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ *
+ *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
+ *    its contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+ * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package com.jgoodies.forms.layout;
@@ -37,15 +37,14 @@ import javax.swing.JPanel;
 import junit.framework.TestCase;
 
 /**
- * Tests the FormLayout's layout algorithm. 
+ * Tests the FormLayout's layout algorithm.
  *
  * @author Karsten Lentzsch
- * @version $Revision: 1.9 $
+ * @version $Revision: 1.14 $
  */
-
 public final class FormLayoutTest extends TestCase {
-    
-    private CellConstraints cc = new CellConstraints();
+
+    private final CellConstraints cc = new CellConstraints();
 
 
     /**
@@ -55,7 +54,7 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "1px, 2px, 3px, 5px, 7px",
             "1px, 2px, 3px");
-            
+
         JPanel panel = new JPanel(layout);
         panel.doLayout();
         FormLayout.LayoutInfo info = layout.getLayoutInfo(panel);
@@ -68,8 +67,8 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Column 4", 11, info.columnOrigins[4]);
         assertEquals("Column 5", 18, info.columnOrigins[5]);
     }
-    
-    
+
+
     /**
      * Checks whether components are aligned according to the column specs.
      */
@@ -82,16 +81,16 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "left:10px, center:10px, right:10px, fill:10px, 10px",
             "pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(left,   cc.xy(1, 1));
         panel.add(center, cc.xy(2, 1));
         panel.add(right,  cc.xy(3, 1));
         panel.add(fill,   cc.xy(4, 1));
         panel.add(def,    cc.xy(5, 1));
-        
+
         panel.doLayout();
-        
+
         assertEquals("Left.x",         0, left.getX());
         assertEquals("Left.width",     4, left.getWidth());
         assertEquals("Center.x",      13, center.getX());
@@ -103,8 +102,8 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Default.x",     40, def.getX());
         assertEquals("Default.width", 10, def.getWidth());
     }
-    
-    
+
+
     /**
      * Checks whether components are aligned according to the row specs.
      */
@@ -117,16 +116,16 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "pref",
             "top:10px, center:10px, bottom:10px, fill:10px, 10px");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(top,     cc.xy(1, 1));
         panel.add(center,  cc.xy(1, 2));
         panel.add(bottom,  cc.xy(1, 3));
         panel.add(fill,    cc.xy(1, 4));
         panel.add(def,     cc.xy(1, 5));
-        
+
         panel.doLayout();
-        
+
         assertEquals("Top.y",           0, top.getY());
         assertEquals("Top.height",      4, top.getHeight());
         assertEquals("Center.y",       13, center.getY());
@@ -138,7 +137,7 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Default.y",      43, def.getY());
         assertEquals("Default.height",  4, def.getHeight());
     }
-    
+
 
     /**
      * Tests bounded min and pref widths.
@@ -153,12 +152,12 @@ public final class FormLayoutTest extends TestCase {
         TestComponent c7 = new TestComponent( 2, 7,  4, 9);
         TestComponent c8 = new TestComponent(20, 7, 40, 9);
         FormLayout layout = new FormLayout(
-            "max(10px;min),  max(10px;min),  " +
-            "max(10px;pref), max(10px;pref), " +
-            "min(10px;min),  min(10px;min),  " +
-            "min(10px;pref), min(10px;pref)",
+            "[10px,min],  [10px,min],  " +
+            "[10px,pref], [10px,pref], " +
+            "[min,10px],  [min,10px],  " +
+            "[pref,10px], [pref,10px]",
             "pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy(1, 1));
         panel.add(c2, cc.xy(2, 1));
@@ -168,9 +167,9 @@ public final class FormLayoutTest extends TestCase {
         panel.add(c6, cc.xy(6, 1));
         panel.add(c7, cc.xy(7, 1));
         panel.add(c8, cc.xy(8, 1));
-        
+
         panel.doLayout();
-        
+
         assertEquals("max(10px;c1_min).width",   10, c1.getWidth());
         assertEquals("max(10px;c2_min).width",   20, c2.getWidth());
         assertEquals("max(10px;c3_pref).width",  10, c3.getWidth());
@@ -180,7 +179,7 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("min(10px;c7_pref).width",   4, c7.getWidth());
         assertEquals("min(10px;c8_pref).width",  10, c8.getWidth());
     }
-    
+
     /**
      * Tests bounded min and pref widths.
      */
@@ -195,11 +194,11 @@ public final class FormLayoutTest extends TestCase {
         TestComponent c8 = new TestComponent(7, 20, 9, 40);
         FormLayout layout = new FormLayout(
             "pref",
-            "f:max(10px;min),  f:max(10px;min),  " +
-            "f:max(10px;pref), f:max(10px;pref), " +
-            "f:min(10px;min),  f:min(10px;min),  " +
-            "f:min(10px;pref), f:min(10px;pref)");
-            
+            "f:[10px,min],  f:[10px,min],  " +
+            "f:[10px,pref], f:[10px,pref], " +
+            "f:[min,10px],  f:[min,10px],  " +
+            "f:[pref,10px], f:[pref,10px]");
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy(1, 1));
         panel.add(c2, cc.xy(1, 2));
@@ -209,22 +208,22 @@ public final class FormLayoutTest extends TestCase {
         panel.add(c6, cc.xy(1, 6));
         panel.add(c7, cc.xy(1, 7));
         panel.add(c8, cc.xy(1, 8));
-        
+
         panel.doLayout();
-        
-        assertEquals("max(10px;c1_min).height",   10, c1.getHeight());
-        assertEquals("max(10px;c2_min).height",   20, c2.getHeight());
-        assertEquals("max(10px;c3_pref).height",  10, c3.getHeight());
-        assertEquals("max(10px;c4_pref).height",  40, c4.getHeight());
-        assertEquals("min(10px;c5_min).height",    2, c5.getHeight());
-        assertEquals("min(10px;c6_min).height",   10, c6.getHeight());
-        assertEquals("min(10px;c7_pref).height",   4, c7.getHeight());
-        assertEquals("min(10px;c8_pref).height",  10, c8.getHeight());
+
+        assertEquals("[10px, c1_min].height",  10, c1.getHeight());
+        assertEquals("[10px, c2_min].height",  20, c2.getHeight());
+        assertEquals("[10px,c3_pref].height",  10, c3.getHeight());
+        assertEquals("[10px,c4_pref].height",  40, c4.getHeight());
+        assertEquals("[c5_min, 10px].height",   2, c5.getHeight());
+        assertEquals("[c6_min, 10px].height",  10, c6.getHeight());
+        assertEquals("[c7_pref,10px].height",   4, c7.getHeight());
+        assertEquals("[c8_pref,10px].height",  10, c8.getHeight());
     }
-    
-    
+
+
     // Testing components that span multiple columns/rows *********************
-    
+
     /**
      * Checks and verifies that components that span multiple columns
      * do not expand the container of no column grows.
@@ -237,7 +236,7 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "10px, 15px, 20px",
             "pref, pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy (1, 1));
         panel.add(c2, cc.xy (2, 1));
@@ -250,12 +249,12 @@ public final class FormLayoutTest extends TestCase {
         int col1And2Width = c2.getX() + c2.getWidth();
         int gridWidth     = c3.getX() + c3.getWidth();
         int totalWidth    = preferredLayoutSize.width;
-        
+
         assertEquals("Col1+2 width", 25, col1And2Width);
         assertEquals("Grid width",   45, gridWidth);
         assertEquals("Total width",  45, totalWidth);
     }
-    
+
 
     /**
      * Checks and verifies that components that span multiple columns
@@ -269,25 +268,25 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "10px, 15px, 20px:grow",
             "pref, pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy (1, 1));
         panel.add(c2, cc.xy (2, 1));
         panel.add(c3, cc.xy (3, 1));
         panel.add(c4, cc.xyw(1, 2, 2));
-        
+
         Dimension preferredLayoutSize = layout.preferredLayoutSize(panel);
         panel.setSize(preferredLayoutSize);
         panel.doLayout();
         int col1And2Width = c2.getX() + c2.getWidth();
         int gridWidth     = c3.getX() + c3.getWidth();
         int totalWidth    = preferredLayoutSize.width;
-        
+
         assertEquals("Col1+2 width",  25, col1And2Width);
         assertEquals("Grid width",    45, gridWidth);
         assertEquals("Total width",   45, totalWidth); // 70 is wrong
     }
-    
+
 
     /**
      * Checks and verifies that components that span multiple columns
@@ -301,25 +300,25 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "10px, 15px:grow, 20px",
             "pref, pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy (1, 1));
         panel.add(c2, cc.xy (2, 1));
         panel.add(c3, cc.xy (3, 1));
         panel.add(c4, cc.xyw(1, 2, 2));
-        
+
         Dimension preferredLayoutSize = layout.preferredLayoutSize(panel);
         panel.setSize(preferredLayoutSize);
         panel.doLayout();
         int col1And2Width = c2.getX() + c2.getWidth();
         int gridWidth     = c3.getX() + c3.getWidth();
         int totalWidth    = preferredLayoutSize.width;
-        
+
         assertEquals("Col1+2 width",  50, col1And2Width);
         assertEquals("Grid width",    70, gridWidth);
-        assertEquals("Total width",   70, totalWidth); 
+        assertEquals("Total width",   70, totalWidth);
     }
-    
+
 
     /**
      * Checks and verifies that components that span multiple columns
@@ -333,20 +332,20 @@ public final class FormLayoutTest extends TestCase {
         FormLayout layout = new FormLayout(
             "10px, 15px:grow, 20px",
             "pref, pref");
-            
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy (1, 1));
         panel.add(c2, cc.xy (2, 1));
         panel.add(c3, cc.xy (3, 1));
         panel.add(c4, cc.xyw(1, 2, 2));
-        
+
         int minimumLayoutWidth   = layout.minimumLayoutSize(panel).width;
         int preferredLayoutWidth = layout.preferredLayoutSize(panel).width;
-        
+
         assertEquals("Minimum layout width",   45, minimumLayoutWidth);
         assertEquals("Preferred layout width", 70, preferredLayoutWidth);
     }
-    
+
 
     /**
      * Tests the layout size, column and row sizes for a default specs.
@@ -354,19 +353,19 @@ public final class FormLayoutTest extends TestCase {
     public void testDefaultSize() {
         TestComponent c1 = new TestComponent(10, 10, 50, 50);
         FormLayout layout = new FormLayout(
-                "default", 
+                "default",
                 "default");
-        
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy(1, 1));
-        
+
         Dimension minimumLayoutSize    = layout.minimumLayoutSize(panel);
         Dimension preferredLayoutSize  = layout.preferredLayoutSize(panel);
         assertEquals("Minimum layout width", 10, minimumLayoutSize.width);
         assertEquals("Minimum layout height", 10, minimumLayoutSize.height);
         assertEquals("Preferred layout width",  50, preferredLayoutSize.width);
         assertEquals("Preferred layout height", 50, preferredLayoutSize.height);
-        
+
         panel.setSize(minimumLayoutSize);
         panel.doLayout();
         int columnWidth = c1.getWidth();
@@ -381,7 +380,7 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Column width (container pref)", 50, columnWidth);
         assertEquals("Row height (container pref)", 50, rowHeight);
     }
-    
+
 
     /**
      * Tests the combination of a default size spec with a lower bound
@@ -390,20 +389,20 @@ public final class FormLayoutTest extends TestCase {
     public void testDefaultWithLowerBound() {
         TestComponent c1 = new TestComponent(10, 10, 50, 50);
         FormLayout layout = new FormLayout(
-                "max(20px;default)", 
-                "max(20px;default)");
-        
+                "[20px,default]",
+                "[20px,default]");
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy(1, 1));
-        
+
         Dimension minimumLayoutSize    = layout.minimumLayoutSize(panel);
         Dimension preferredLayoutSize  = layout.preferredLayoutSize(panel);
-        
+
         assertEquals("Minimum layout width", 20, minimumLayoutSize.width);
         assertEquals("Minimum layout height", 20, minimumLayoutSize.height);
         assertEquals("Preferred layout width",  50, preferredLayoutSize.width);
         assertEquals("Preferred layout height", 50, preferredLayoutSize.height);
-        
+
         panel.setSize(minimumLayoutSize);
         panel.doLayout();
         int columnWidth = c1.getWidth();
@@ -418,7 +417,7 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Column width (container pref)", 50, columnWidth);
         assertEquals("Row height (container pref)", 50, rowHeight);
     }
-    
+
 
     /**
      * Tests the combination of a default size spec with an upper bound
@@ -427,20 +426,20 @@ public final class FormLayoutTest extends TestCase {
     public void testDefaultWithUpperBound() {
         TestComponent c1 = new TestComponent(10, 10, 50, 50);
         FormLayout layout = new FormLayout(
-                "min(20px;default)", 
-                "min(20px;default)");
-        
+                "[default,20px]",
+                "[default,20px]");
+
         JPanel panel = new JPanel(layout);
         panel.add(c1, cc.xy(1, 1));
-        
+
         Dimension minimumLayoutSize    = layout.minimumLayoutSize(panel);
         Dimension preferredLayoutSize  = layout.preferredLayoutSize(panel);
-        
+
         assertEquals("Minimum layout width", 10, minimumLayoutSize.width);
         assertEquals("Minimum layout height", 10, minimumLayoutSize.height);
         assertEquals("Preferred layout width",  20, preferredLayoutSize.width);
         assertEquals("Preferred layout height", 20, preferredLayoutSize.height);
-        
+
         panel.setSize(minimumLayoutSize);
         panel.doLayout();
         int columnWidth = c1.getWidth();
@@ -455,6 +454,61 @@ public final class FormLayoutTest extends TestCase {
         assertEquals("Column width (container pref)", 20, columnWidth);
         assertEquals("Row height (container pref)", 20, rowHeight);
     }
-    
 
-}
\ No newline at end of file
+
+    // Visibility *************************************************************
+
+    /**
+     * Checks whether components are taken into account according
+     * to their visibility, the container wide FormLayout setting,
+     * and the cell setting.
+     */
+    public void testVisibility() {
+        testVisibility(true);
+        testVisibility(false);
+    }
+
+
+    public void testVisibility(boolean containerHonorsVisibility) {
+        TestComponent visible   = new TestComponent(10, 10, 10, 10);
+        TestComponent invisible = new TestComponent(10, 10, 10, 10);
+        invisible.setVisible(false);
+        TestComponent invisibleHonorsVisibility = new TestComponent(10, 10, 10, 10);
+        invisibleHonorsVisibility.setVisible(false);
+        TestComponent invisibleIgnoresVisibility = new TestComponent(10, 10, 10, 10);
+        invisibleIgnoresVisibility.setVisible(false);
+        FormLayout layout = new FormLayout(
+            "pref, pref, pref, pref",
+            "pref, pref, pref, pref");
+        layout.setHonorsVisibility(containerHonorsVisibility);
+
+        JPanel panel = new JPanel(layout);
+        panel.add(visible,                    cc.xy(1, 1));
+        panel.add(invisible,                  cc.xy(2, 2));
+        panel.add(invisibleHonorsVisibility,  cc.xy(3, 3));
+        panel.add(invisibleIgnoresVisibility, cc.xy(4, 4));
+        layout.setHonorsVisibility(invisibleHonorsVisibility, Boolean.TRUE);
+        layout.setHonorsVisibility(invisibleIgnoresVisibility, Boolean.FALSE);
+
+        panel.doLayout();
+        FormLayout.LayoutInfo info = layout.getLayoutInfo(panel);
+        int size1 = 10;
+        int size2 = containerHonorsVisibility ? 0 : 10;
+        int size3 = 0;
+        int size4 = 10;
+        int origin1 = size1;
+        int origin2 = origin1 + size2;
+        int origin3 = origin2 + size3;
+        int origin4 = origin3 + size4;
+        assertEquals("Column 0",  0,       info.columnOrigins[0]);
+        assertEquals("Column 1",  origin1, info.columnOrigins[1]);
+        assertEquals("Column 2",  origin2, info.columnOrigins[2]);
+        assertEquals("Column 3",  origin3, info.columnOrigins[3]);
+        assertEquals("Column 4",  origin4, info.columnOrigins[4]);
+        assertEquals("Row 0",     0,       info.rowOrigins[0]);
+        assertEquals("Row 1",     origin1, info.rowOrigins[1]);
+        assertEquals("Row 2",     origin2, info.rowOrigins[2]);
+        assertEquals("Row 3",     origin3, info.rowOrigins[3]);
+        assertEquals("Row 4",     origin4, info.rowOrigins[4]);
+    }
+}
